unit Utils;

interface

uses Forms, Graphics, ShellAPI, Windows, SysUtils;

type
  TFormPos  = Record
    ws:       TWindowState;
    top,
    left,
    width,
    height:   Integer;
  end;

const
	sRegKeyRoot = 'Software\Tech-Pro\MorseGen';
	Version = 'Version 1.00';


function GetStringFromRegistry(const AppName, KeyName, Default: String): String;
procedure WriteStringToRegistry(const AppName, KeyName, Value: String);
function GetIntFromRegistry(const AppName, KeyName: String; Default: Integer): Integer;
procedure WriteIntToRegistry(const AppName, KeyName: String; Value: Integer);
procedure GetBinFromRegistry(const AppName, KeyName: String; P: Pointer; len: Integer);
procedure WriteBinToRegistry(const AppName, KeyName: String; P: Pointer; len: Integer);

procedure GetFormPosFromRegistry(const AppName, KeyName: String; AForm: TForm; getsize: Boolean);
procedure WriteFormPosToRegistry(const AppName, KeyName: String; AForm: TForm; savesize: Boolean);

function GetRootKey(const KeyName, ValueName, Default: String): String;
procedure SetRootKey(const KeyName, ValueName, Value: String);
function GetRootKeys(const KeyName: String): String;

implementation

function GetStringFromRegistry(const AppName, KeyName, Default: String): String;
var
	newkey: HKey;
  keytype, keysize: DWord;
  buff: Array[0..MAX_PATH] of Char;
begin
	Result := Default;
  if RegOpenKeyEx(HKEY_CURRENT_USER,PChar(AppName),0,KEY_ALL_ACCESS,newkey) = ERROR_SUCCESS then
  begin
  	keysize := MAX_PATH;
  	RegQueryValueEx(newkey,PChar(Keyname),LPDWORD(0),@keytype,@buff,@keysize);
		case keytype of
      REG_SZ:			Result := StrPas(PChar(@buff[0]));
      REG_DWORD:	Result := IntToStr(DWORD(buff[0]));
    end;
		RegCloseKey(newkey);
  end;
end;

procedure WriteStringToRegistry(const AppName, KeyName, Value: String);
var
  d: Integer;
	newkey: HKey;
begin
  if RegCreateKeyEx(HKEY_CURRENT_USER,PChar(AppName),0,nil,REG_OPTION_NON_VOLATILE,KEY_ALL_ACCESS,nil,newkey,@d) = ERROR_SUCCESS then
  begin
  	RegSetValueEx(newkey,PChar(Keyname),0,REG_SZ,PChar(Value),Length(Value)+1);
		RegCloseKey(newkey);
  end;
end;

function GetIntFromRegistry(const AppName, KeyName: String; Default: Integer): Integer;
var
	newkey: HKey;
  keytype, keysize: DWord;
  intresult: Integer;
begin
	Result := Default;
  if RegOpenKeyEx(HKEY_CURRENT_USER,PChar(AppName),0,KEY_ALL_ACCESS,newkey) = ERROR_SUCCESS then
  begin
  	keysize := 4;
  	RegQueryValueEx(newkey,PChar(Keyname),LPDWORD(0),@keytype,@intresult,@keysize);
		if keytype = REG_DWORD then
    	Result := intresult;
		RegCloseKey(newkey);
  end;
end;

procedure WriteIntToRegistry(const AppName, KeyName: String; Value: Integer);
var
  d: Integer;
	newkey: HKey;
begin
  if RegCreateKeyEx(HKEY_CURRENT_USER,PChar(AppName),0,nil,REG_OPTION_NON_VOLATILE,KEY_ALL_ACCESS,nil,newkey,@d) = ERROR_SUCCESS then
  begin
  	RegSetValueEx(newkey,PChar(Keyname),0,REG_DWORD,@Value,4);
		RegCloseKey(newkey);
  end;
end;

procedure GetBinFromRegistry(const AppName, KeyName: String; P: Pointer; len: Integer);
var
	newkey: HKey;
  keytype, keysize: DWord;
begin
  FillChar(P^, len, 0);
  if RegOpenKeyEx(HKEY_CURRENT_USER,PChar(AppName),0,KEY_READ,newkey) = ERROR_SUCCESS then
  begin
  	keysize := len;
  	RegQueryValueEx(newkey,PChar(Keyname),LPDWORD(0),@keytype,P,@keysize);
		RegCloseKey(newkey);
  end;
end;

procedure WriteBinToRegistry(const AppName, KeyName: String; P: Pointer; len: Integer);
var
  d: Integer;
	newkey: HKey;
begin
  if RegCreateKeyEx(HKEY_CURRENT_USER,PChar(AppName),0,nil,REG_OPTION_NON_VOLATILE,KEY_ALL_ACCESS,nil,newkey,@d) = ERROR_SUCCESS then
  begin
  	RegSetValueEx(newkey,PChar(Keyname),0,REG_BINARY,P,len);
		RegCloseKey(newkey);
  end;
end;

procedure GetFormPosFromRegistry(const AppName, KeyName: String; AForm: TForm; getsize: Boolean);
var
  FormPos: TFormPos;
begin
  FillChar(FormPos,SizeOf(FormPos),0);
  GetBinFromRegistry(AppName, KeyName, @FormPos, SizeOf(FormPos));
  with FormPos do
    if (height > 40) then
    begin
      AForm.Top := top;
      if left < Screen.Width then AForm.Left := left;
      if getsize then
      begin
        if width > 40 then AForm.Width := width;
        AForm.Height := height;
      end;
      Application.ProcessMessages;
    	if ws = wsMaximized then AForm.WindowState := ws;
		end;
end;

procedure WriteFormPosToRegistry(const AppName, KeyName: String; AForm: TForm; savesize: Boolean);
var
  FormPos: TFormPos;
begin
  GetBinFromRegistry(AppName, KeyName, @FormPos, SizeOf(FormPos));
	with FormPos do
  begin
    ws := AForm.WindowState;
    if ws = wsNormal then
    begin
      top := AForm.Top;
      left := AForm.Left;
      width := AForm.Width;
      height := AForm.Height;
    end;
  end;
  WriteBinToRegistry(AppName, KeyName, @FormPos, SizeOf(FormPos));
end;

function GetRootKey(const KeyName, ValueName, Default: String): String;
var
	newkey: HKey;
  keytype, keysize: DWord;
  buff: Array[0..255] of Char;
begin
	Result := Default;
	if RegOpenKey(HKEY_CLASSES_ROOT,PChar(KeyName),newkey) = 0 then
  begin
  	keysize := 256;
  	RegQueryValueEx(newkey,PChar(ValueName),LPDWORD(0),@keytype,@buff,@keysize);
		Result := StrPas(PChar(@buff[0]));
		RegCloseKey(newkey);
  end;
end;

function GetRootKeys(const KeyName: String): String;
var
	newkey: HKey;
  indx, keysize: DWord;
  buff: Array[0..255] of Char;
begin
	Result := '';
	if RegOpenKey(HKEY_CLASSES_ROOT,PChar(KeyName),newkey) = 0 then
  begin
  	keysize := 256;
    indx := 0;
    while RegEnumKey(newkey,indx,@buff,keysize) = ERROR_SUCCESS do
    begin
      Result := Result + #13 + StrPas(PChar(@buff[0]));
      inc(indx);
    end;
    Delete(Result,1,1);
		RegCloseKey(newkey);
  end;
end;

procedure SetRootKey(const KeyName, ValueName, Value: String);
var
	newkey: HKey;
begin
	if RegCreateKey(HKEY_CLASSES_ROOT,PChar(KeyName),newkey) = 0 then
  begin
  	RegSetValueEx(newkey,PChar(Valuename),0,REG_SZ,PChar(Value),Length(Value)+1);
		RegCloseKey(newkey);
  end;
end;

end.
